@@ -4,6 +4,7 @@ import os
 import shutil
 import importlib
 import importlib.util
+import pickle
 from copy import deepcopy
 from pathlib import Path
 from typing import Union
@@ -22,6 +23,7 @@ from steam_sdk.utils.make_folder_if_not_existing import make_folder_if_not_exist
 from steam_sdk.utils.rgetattr import rgetattr
 from steam_sdk.utils.sgetattr import rsetattr
 from steam_sdk.viewers.Viewer import Viewer
+from steam_sdk.parsers.ParserPSPICE import writeStimuliFromInterpolation, writeStimuliFromInterpolation_general
 
 
 class AnalysisSTEAM:
@@ -29,7 +31,9 @@ class AnalysisSTEAM:
         Class to run analysis based on STEAM_SDK
     """
 
-    def __init__(self, file_name_analysis: str = None,
+    def __init__(self,
+                 file_name_analysis: str = None,
+                 file_path_list_models: str = '',
                  relative_path_settings: str = '',
                  verbose: bool = False):
         """
@@ -44,12 +48,17 @@ class AnalysisSTEAM:
         self.library_path = None
         self.output_path = None
         self.temp_path = None
-        self.list_models = {}  # this dictionary will be populated with BuilderModel objects and their names
+        if file_path_list_models:
+            with open(file_path_list_models, 'rb') as input_dict:
+                self.list_models = pickle.load(input_dict)
+        else:
+            self.list_models = {}  # this dictionary will be populated with BuilderModel objects and their names
+
         self.list_sims = []  # this list will be populated with integers indicating simulations to run
         self.list_viewers = {}  # this dictionary will be populated with Viewer objects and their names
         self.list_metrics = {}  # this dictionary will be populated with calculated metrics
         self.verbose = verbose
-        self.overall_parameter = None  # float representing the overall outcome of a simulation for parsims
+        self.summary = None  # float representing the overall outcome of a simulation for parsims
 
         if file_name_analysis:
             self.data_analysis = yaml_to_data(file_name_analysis, DataAnalysis)  # Load yaml keys into DataAnalysis dataclass
@@ -159,6 +168,22 @@ class AnalysisSTEAM:
         dict_to_yaml(settings_dict, path_temp_file_settings)
         if verbose: print('File {} was saved locally.'.format(path_temp_file_settings))
 
+    def store_model_objects(self, path_output_file: str):
+        """
+        ** Stores the dictionary of BuilderModel objects in a pickle file at the specified path **
+        This can be helpful to load the list of models instead of generating it at every iteration of a co-simulation.
+        :param path_output_file: string to the file to write
+        :return: None
+        """
+        # Make sure the target folder exists
+        make_folder_if_not_existing(os.path.dirname(path_output_file), verbose=self.verbose)
+
+        # Store the objects as pickle file
+        with open(path_output_file, 'wb') as output:
+            pickle.dump(self.list_models, output, pickle.HIGHEST_PROTOCOL)
+
+        if self.verbose: print(f'File {path_output_file} saved.')
+
     def write_analysis_file(self, path_output_file: str):
         """
         ** Write the analysis data in the target file **
@@ -223,6 +248,10 @@ class AnalysisSTEAM:
                 self.run_viewer(step, verbose=verbose)
             elif step.type == 'CalculateMetrics':
                 self.calculate_metrics(step, verbose=verbose)
+            elif step.type == 'LoadCircuitParameters':
+                self.load_circuit_parameters(step, verbose=verbose)
+            elif step.type == 'WriteStimulusFile':
+                self.write_stimuli_from_interpolation(step, verbose=verbose)
             else:
                 raise Exception('Unknown type of analysis step: {}'.format(step.type))
 
@@ -341,7 +370,8 @@ class AnalysisSTEAM:
                 simulation_number = step.simulation_numbers[v]
                 if 'FiQuS' in step.software:
                     BM.buildFiQuS(simulation_name=f'{step.simulation_name}')
-                    self.setup_sim_FiQuS(simulation_name=step.simulation_name, sim_number=simulation_number)
+                    self.setup_sim_FiQuS(simulation_name=step.simulation_name, sim_number=simulation_number,
+                                         magnet_type=BM.model_data.GeneralParameters.magnet_type if not step.variable_to_change.startswith('Options_FiQuS.') else None)
                 if 'LEDET' in step.software:
                     flag_json = BM.flag_json
                     flag_yaml = True  # Hard-coded for the moment
@@ -471,6 +501,7 @@ class AnalysisSTEAM:
             if verbose:
                 print('Set up folder of model {} for {}.'.format(simulation_name, software))
 
+
             if 'FiQuS' in software:
                 # make top level output folder
                 make_folder_if_not_existing(self.settings.local_FiQuS_folder, verbose=verbose)
@@ -584,7 +615,7 @@ class AnalysisSTEAM:
         if len_simulation_numbers > 0:
             for simulation_number in step.simulation_numbers:
                 if 'FiQuS' in step.software:
-                    self.setup_sim_FiQuS(simulation_name=step.simulation_name, sim_number=simulation_number, simulation_type=step.type)
+                    self.setup_sim_FiQuS(simulation_name=step.simulation_name, sim_number=simulation_number)
                 if 'LEDET' in step.software:
                     flag_yaml = True  # Hard-coded for the moment
                     flag_json = False  # Hard-coded for the moment
@@ -769,6 +800,34 @@ class AnalysisSTEAM:
 
         return current_list_output_metrics
 
+    def load_circuit_parameters(self, step, verbose: bool = False):
+        """
+        Load global circuit parameters from a .csv file into an existing BuilderModel circuit model
+        :param step: STEAM analysis step of type LoadCircuitParameters, which has attributes:
+        - model_name: BuilderModel object to edit - THIS MUST BE OF TYPE CIRCUIT
+        - path_file_circuit_parameters: the name of the .csv file containing the circuit parameters
+        - selected_circuit_name: name of the circuit name whose parameters will be loaded
+        :param verbose: display additional logging info
+        :return:
+        """
+        if verbose: print(f'Load circuit parameters.')
+
+        # Unpack variables
+        model_name = step.model_name
+        path_file_circuit_parameters = step.path_file_circuit_parameters
+        selected_circuit_name = step.selected_circuit_name
+
+        BM=self.list_models[model_name]
+
+        # Call function to load the parameters into the object
+        BM.load_circuit_parameters_from_csv(input_file_name=path_file_circuit_parameters, selected_circuit_name=selected_circuit_name)
+
+        # Update the BuilderModel object
+        self.list_models[model_name] = BM
+
+        return
+
+
     def setup_sim_FiQuS(self, simulation_name, sim_number, magnet_type=None):
         """
         Set up a FiQuS simulation by copying the last file generated by BuilderModel to the output folder and to the
@@ -883,6 +942,21 @@ class AnalysisSTEAM:
             shutil.copyfile(file_to_copy, file_copied)
             if self.verbose: print('Stl file {} copied to {}.'.format(file_to_copy, file_copied))
 
+        # # Copy Conf.cir file from the output folder
+        # list_cir_files = [entry for entry in os.listdir(output_folder) if ("Conf" in entry) and ('.cir' in entry)]
+        # for cir_file in list_cir_files:
+        #     file_to_copy = os.path.join(output_folder, cir_file)
+        #     file_copied = os.path.join(local_model_folder, cir_file)
+        #     shutil.copyfile(file_to_copy, file_copied)
+        #     if self.verbose: print('Cir file {} copied to {}.'.format(file_to_copy, file_copied))
+
+        # Special case: Copy coil_resistances.stl file from the output folder
+        file_coil_resistances_to_copy = os.path.join(output_folder, 'coil_resistances.stl')
+        file_coil_resistances_copied = os.path.join(local_model_folder, 'coil_resistances.stl')
+        if os.path.isfile(file_coil_resistances_to_copy):
+            shutil.copyfile(file_coil_resistances_to_copy, file_coil_resistances_copied)
+            if self.verbose: print('Stl file {} copied to {}.'.format(file_coil_resistances_to_copy, file_coil_resistances_copied))
+
     def setup_sim_XYCE(self, simulation_name, sim_number):
         """
         Set up a PSPICE simulation by copying the last file generated by BuilderModel to the output folder and to the
@@ -907,7 +981,7 @@ class AnalysisSTEAM:
         file_name_local = os.path.join(local_model_folder, simulation_name + '.cir')
         if self.verbose: print('Simulation file {} generated.'.format(file_name_temp))
 
-        shutil.copyfile(file_name_temp, file_name_local)
+        self.copy_XYCE_cir(file_name_temp, file_name_local)
         if self.verbose: print('Simulation file {} copied.'.format(file_name_local))
 
         # os.remove(file_name_temp)
@@ -927,20 +1001,55 @@ class AnalysisSTEAM:
             shutil.copyfile(file_to_copy, file_copied)
             if self.verbose: print('Lib file {} copied to {}.'.format(file_to_copy, file_copied))
 
-        # Copy stl files from the output folder
-        stl_path = os.path.join(self.output_path, 'Stimulus')
-        stl_path_new = os.path.join(local_model_folder, 'Stimulus')
-        if os.path.exists(stl_path_new):
-            shutil.rmtree(stl_path_new)
-        os.mkdir(stl_path_new)
+        # Copy csv files from the output folder
+        csv_path = os.path.join(self.output_path, 'Stimulus')
+        csv_path_new = os.path.join(local_model_folder, 'Stimulus')
+        if os.path.exists(csv_path_new):
+            shutil.rmtree(csv_path_new)
+        os.mkdir(csv_path_new)
 
-        list_stl_files = [entry for entry in os.listdir(stl_path) if
-                          (simulation_name in entry) and ('.csv' in entry)]
-        for stl_file in list_stl_files:
-            file_to_copy = os.path.join(stl_path, stl_file) #self.output_path was first argument; changed it to stl_path
-            file_copied = os.path.join(stl_path_new, stl_file)
+        list_csv_files = [entry for entry in os.listdir(csv_path) if (simulation_name in entry) and ('.csv' in entry)]
+        for csv_file in list_csv_files:
+            file_to_copy = os.path.join(csv_path, csv_file) #self.output_path was first argument; changed it to stl_path
+            file_copied = os.path.join(csv_path_new, csv_file)
             shutil.copyfile(file_to_copy, file_copied)
-            if self.verbose: print('Stl file {} copied to {}.'.format(file_to_copy, file_copied))
+            if self.verbose: print('Csv file {} copied to {}.'.format(file_to_copy, file_copied))
+
+        # Special case: Copy coil_resistances.csv file from the output folder
+        file_coil_resistances_to_copy = os.path.join(output_folder, 'coil_resistances.csv')
+        file_coil_resistances_copied = os.path.join(local_model_folder, 'coil_resistances.csv')
+        if os.path.isfile(file_coil_resistances_to_copy):
+            shutil.copyfile(file_coil_resistances_to_copy, file_coil_resistances_copied)
+            if self.verbose: print('Csv file {} copied to {}.'.format(file_coil_resistances_to_copy, file_coil_resistances_copied))
+
+    def copy_XYCE_cir(self, file_name_temp, file_name_local):
+        '''
+            Function that copies the XYCE circuit file from 'file_name_temp' to 'file_name_local' and changes the
+            respective output path for the csd
+        :param file_name_temp: Original circuit file
+        :param file_name_local: Final circuit file
+        :return:
+        '''
+
+        with open(file_name_temp) as f:
+            contents = f.readlines()
+        for k in range(len(contents)):
+            if contents[k].casefold().startswith('.print'):
+                if 'csd' in contents[k]: type_output = 'csd'
+                elif 'csv' in contents[k]: type_output = 'csv'
+                elif 'txt' in contents[k]: type_output = 'txt'
+                else:
+                    raise Exception("Don't understand output type.")
+                print_line = contents[k].split('FILE=')
+                print_line[-1] = f'FILE={file_name_local[:-4]}.{type_output} \n'
+                contents[k] = ''.join(print_line)
+                break
+
+        contents = ''.join(contents)
+        new_file = open(file_name_local, 'w')
+        new_file.write(contents)
+        new_file.close()
+
 
     def setup_sim_PyBBQ(self, simulation_name, sim_number):
         """
@@ -971,6 +1080,7 @@ class AnalysisSTEAM:
         # os.remove(file_name_temp)
         # print('Temporary file {} deleted.'.format(file_name_temp))
 
+
     def run_sim(self, software: str, simulation_name: str, sim_number: int, simFileType: str = None, verbose: bool = False):
         """
         Run selected simulation.
@@ -980,7 +1090,7 @@ class AnalysisSTEAM:
             local_analysis_folder = simulation_name + '_' + str(sim_number)
             dFiQuS = DriverFiQuS(FiQuS_path=self.settings.FiQuS_path, path_folder_FiQuS=self.settings.local_FiQuS_folder,
                                  path_folder_FiQuS_input=os.path.join(self.settings.local_FiQuS_folder, local_analysis_folder), verbose=verbose)
-            self.overall_parameter = dFiQuS.run_FiQuS(sim_file_name=simulation_name, output_directory=local_analysis_folder)
+            self.summary = dFiQuS.run_FiQuS(sim_file_name=simulation_name, output_directory=local_analysis_folder)
         elif software == 'LEDET':
             dLEDET = DriverLEDET(path_exe=self.settings.LEDET_path, path_folder_LEDET=self.settings.local_LEDET_folder, verbose=verbose)
             dLEDET.run_LEDET(simulation_name, str(sim_number), simFileType=simFileType)
@@ -999,3 +1109,41 @@ class AnalysisSTEAM:
             dXYCE.run_XYCE(simulation_name, suffix='')
         else:
             raise Exception(f'Software {software} not supported for automated running.')
+
+
+    def write_stimuli_from_interpolation(self, step, verbose: bool = False):
+        '''
+        Function to write a resistance stimuli for n apertures of a magnet for any current level. Resistance will be interpolated
+        from pre-calculated values (see InterpolateResistance for closer explanation). Stimuli is then written in a .stl file for PSPICE
+
+        :param current_level: list, all current level that shall be used for interpolation (each magnet has 1 current level)
+        :param n_total_magnets: int, Number of total magnets in the circuit (A stimuli will be written for each, non-quenching = 0)
+        :param n_apertures: int, Number of apertures per magnet. A stimuli will be written for each aperture for each magnet
+        :param magnets: list, magnet numbers for which the stimuli shall be written
+        :param tShift: list, time shift that needs to be applied to each stimuli
+        (e.g. if magnet 1 quenches at 0.05s, magnet 2 at 1s etc.), so that the stimuli are applied at the correct time in the simulation
+        :param Outputfile: str, name of the stimuli-file
+        :param path_resources: str, path to the file with pre-calculated values
+        :param Type: str, either Linear or Spline, type of interpolation
+        :param type_stl: str, how to write the stimuli file (either 'a' (append) or 'w' (write))
+        :param sparseTimeStepping: int, every x-th time value only a stimuli point is written (to reduce size of stimuli)
+        :return:
+        '''
+        n_apertures     = step.n_apertures
+        current_level   = step.current_level
+        n_total_magnets = step.n_total_magnets
+        magnets         = step.magnets
+        tShift          = step.t_offset
+        Outputfile      = step.output_file
+        path_resources  = step.path_interpolation_file
+        magnet_type = step.magnet_types
+
+        # Hard-coded settings
+        InterpolationType = 'Linear'
+        type_stl = 'w'
+        sparseTimeStepping = 100
+
+        writeStimuliFromInterpolation_general(current_level, n_total_magnets, n_apertures, magnets, tShift, Outputfile, path_resources, InterpolationType, type_stl, sparseTimeStepping, magnet_type)
+
+        if verbose:
+            print(f'Output stimulus file {Outputfile} written.')
