@@ -43,7 +43,7 @@ class BuilderFiQuS:
             else:
                 raise Exception(f'Magnet type: {model_data.GeneralParameters.magnet_type} is incompatible with FiQuS.')
 
-    def buildDataCCT(self, model_data):
+    def buildDataCCT(self, model_data): #TODO: good idea to make them private?
         """
             Load selected conductor data from DataModelMagnet keys, check inputs, calculate and set missing variables
         """
@@ -129,4 +129,3 @@ class BuilderFiQuS:
 
         # ------------- postproc ---------
         self.data_FiQuS.magnet.postproc = model_data.Options_FiQuS.multipole.postproc
-        self.data_FiQuS.magnet.postproc.plot_all = flag_plot_all
