@@ -1,6 +1,7 @@
 import os
 import shutil
 from pathlib import Path
+import pandas as pd
 
 from steam_sdk.builders.BuilderPyBBQ import BuilderPyBBQ
 from steam_sdk.data.DataModelConductor import DataModelConductor
@@ -11,6 +12,7 @@ from steam_sdk.builders.BuilderFiQuS import BuilderFiQuS
 from steam_sdk.builders.BuilderLEDET import BuilderLEDET
 from steam_sdk.builders.BuilderProteCCT import BuilderProteCCT
 from steam_sdk.builders.BuilderSIGMA import BuilderSIGMA
+from steam_sdk.parsers.ParserCsv import find_by_position, find_column_list
 from steam_sdk.parsers.ParserFiQuS import ParserFiQuS
 from steam_sdk.parsers.ParserLEDET import ParserLEDET, copy_modified_map2d_ribbon_cable, copy_map2d
 from steam_sdk.parsers.ParserPSPICE import ParserPSPICE
@@ -38,8 +40,17 @@ class BuilderModel:
         """
             Builder object to generate models from STEAM simulation tools specified by user
 
-            output_path: path to the output models
+            file_model_data: path to folder with input data (roxie files, geometry, modelData.yaml = config from userinput)
+            software: list of simulation software to be used
+            case_model: defines whether it is a magnet, a conductor or a circuit
+            relative_path_settings: path to user specific settings.SYSTEM.yaml file
+            flag_build: defines whether the model has to be build (so whether output folder should be created) - but why ???
+            flag_dump_all: ????
+            flag_json: ????
             dump_all_path: path to the final yaml file
+            output_path: path to the output models
+            verbose: to display internal processes (output of status & error messages) for troubleshooting
+            flag_plot_all: defines whether the tools in steam framework should show the figrues
         """
 
         # Unpack arguments
@@ -69,7 +80,7 @@ class BuilderModel:
 
         ### Case of a magnet model
         if self.case_model == 'magnet':
-            # Initialize
+            # Initialized
             self.model_data: DataModelMagnet = DataModelMagnet()
             self.roxie_data: RoxieData = RoxieData()
             self.path_magnet = None
@@ -98,7 +109,7 @@ class BuilderModel:
                     if s == 'FiQuS':    self.buildFiQuS()
                     if s == 'LEDET':    self.buildLEDET()
                     if s == 'ProteCCT': self.buildProteCCT()
-                    if s == 'SIGMA':    self.buildSIGMA()
+                    if s == 'SIGMA':    self.buildSIGMA()   # TODO: raise error when wrong software selected?
 
                 if flag_dump_all:
                     self.dumpAll()
@@ -120,7 +131,7 @@ class BuilderModel:
                     print('output_path: {}'.format(self.output_path))
 
                 # Load model data from the input .yaml file
-                self.loadModelData()
+                self.loadModelData()  #TODO consider if this should be moved before the "if flag_build" statement
 
                 # Set paths of input files and settings
                 self.set_input_paths()
@@ -129,6 +140,8 @@ class BuilderModel:
                 for s in self.software:
                     if s == 'PSPICE': self.buildPSPICE()
                     elif s == 'XYCE': self.buildXYCE()
+                    else:
+                        raise Exception('Please specify software (PSPICE/XYCE).')
 
         ### Case of a conductor model
         elif self.case_model == 'conductor':
@@ -150,11 +163,11 @@ class BuilderModel:
                 # Build model
                 for s in self.software:
                     if s == 'LEDET':    self.buildLEDET()
-                    if s == 'PyBBQ':    self.buildPyBBQ()
+                    if s == 'PyBBQ':    self.buildPyBBQ() # TODO: raise error when wrong software selected?
 
     def set_input_paths(self):
         """
-            Sets input paths and displays related information
+            Sets input paths from created DataModel, set path to usersettings and displays related information
         """
         # TODO: Add test for this method
 
@@ -190,7 +203,7 @@ class BuilderModel:
         # Set settings path
         # Find user name
         user_name = 'user'  # till GitLab path problem is solved
-        for environ_var in ['HOMEPATH', 'SWAN_HOME']:
+        for environ_var in ['HOMEPATH', 'SWAN_HOME']:  # TODO: why not use os.getlogin() as in AnalysisSTEAM.py
             if environ_var in os.environ:
                 user_name = os.path.basename(os.path.normpath(os.environ[environ_var]))
         if self.verbose:
@@ -240,6 +253,8 @@ class BuilderModel:
                     Parser = ParserXYCE(self.circuit_data)
                     Parser.readFromYaml(self.file_model_data)
                     self.circuit_data = Parser.circuit_data
+                else:
+                    raise Exception('Please specify software (PSPICE/XYCE).')
         elif self.case_model == 'conductor':
             # Load yaml keys into DataModelConductor dataclass
             self.conductor_data = yaml_to_data(self.file_model_data, DataModelConductor)
@@ -513,6 +528,40 @@ class BuilderModel:
                      settings_dict=self.settings_dict, output_path=self.output_path,
                      flag_build=self.flag_build, verbose=self.verbose)
 
+
+    def load_circuit_parameters_from_csv(self, input_file_name: str, selected_circuit_name: str = None):
+        '''
+            Load circuit parameters from a csv file into a case=circuit object
+            :param input_file_name: full path to the csv file
+            :param selected_circuit_name: name of the circuit name whose parameters will be loaded
+        '''
+
+        # Unpack input
+        verbose = self.verbose
+
+        # Check that the model case is circuit
+        if self.case_model != 'circuit':
+            raise Exception(f'This method can only be used for circuit models, but case_model={self.case_model}')
+
+        # TODO: Think on whether this is good to add as a default option
+        # if selected_circuit_name is None:
+        #     selected_circuit_name = self.circuit_data.GeneralParameters.circuit_name
+
+        circuit_param = find_by_position(input_file_name, 0, selected_circuit_name)  # TODO make it more robust by referring to a key rather than the first column
+        #del circuit_param[0]
+        param_names = find_column_list(input_file_name)
+        #del param_names[0]
+        dict_circuit = {k: v for k, v in zip(param_names, circuit_param)}
+
+        for key, value in dict_circuit.items():
+            if key in self.circuit_data.GlobalParameters.global_parameters:
+                if self.circuit_data.GlobalParameters.global_parameters[key] != value:
+                    if verbose:
+                        print(f'Circuit {self.circuit_data.GeneralParameters.circuit_name}: Value of parameter {key} was changed from {self.circuit_data.GlobalParameters.global_parameters[key]} to {value}')
+                    self.circuit_data.GlobalParameters.global_parameters[key] = value
+
+
+
     def dumpAll(self):
         """
             Writes model data and data from Roxie parser in a combined .yaml file
