@@ -16,12 +16,17 @@ class BuilderSIGMA:
         Class to generate SIGMA models
     """
 
-    def __init__(self, input_model_data: DataModelMagnet = None, input_roxie_data: dR.RoxieData = None,
-                 settings_dict: dict = None, output_path: str = None, flag_build: bool = True, verbose: bool = True):
+    def __init__(self, coordinate_file_path, result_txt_path, input_model_data: DataModelMagnet = None,
+                 input_roxie_data: dR.RoxieData = None,
+                 settings_dict: dict = None, output_path: str = None, flag_build: bool = True, verbose: bool = True,
+                 generate_study=False):
         self.verbose: bool = verbose
+        self.input_coordinates_path = coordinate_file_path
+        self.result_txt_path = result_txt_path
         self.model_data: DataModelMagnet = input_model_data
         self.roxie_data: dR.RoxieData = input_roxie_data
         self.settings_dict: dict = settings_dict
+        self.generate_study = generate_study
         if output_path:
             self.output_path: str = output_path
         else:
@@ -235,7 +240,6 @@ class BuilderSIGMA:
                         currents = ()
                         for block_key, block in winding.blocks.items():
                             currents += (block.current_sign,)
-
                             kp0 = self.g.Point.ofCartesian(coil.bore_center.x, coil.bore_center.y)
                             arg = [self.g.Point.ofCartesian(block.block_corners.iL.x, block.block_corners.iL.y),
                                    self.g.Point.ofCartesian(block.block_corners.iR.x, block.block_corners.iR.y),
@@ -248,8 +252,8 @@ class BuilderSIGMA:
                                                                 self.g.Line.ofEndPoints(arg[1], arg[3]),
                                                                 self.g.Arc.ofEndPointsCenter(arg[3], arg[2], kp0),
                                                                 self.g.Line.ofEndPoints(arg[0], arg[2])))),)
-
                         self.coil_areas += [areas[0], areas[1]]
+
                         windings += (self.g.Winding.ofAreas(self.a.create_area_array(self.g.gateway, areas),
                                                             self.a.create_int_array(self.g.gateway, currents),
                                                             winding.conductors_number, winding.conductors_number,
@@ -325,8 +329,11 @@ class BuilderSIGMA:
         srv = self.g.TxtSigmaServer(cfg.getOutputModelPath(), cfg.getComsolBatchPath())
         srv.connect(cfg.getComsolBatchPath())
         self.model = self.g.MagnetMPHBuilder(cfg, srv)
-
-        self.model.buildMPH(self.a.create_domain_array(self.g.gateway, tuple(domains)))
+        # Create map with I0.
+        variable_map = self.g.gateway.jvm.java.util.HashMap()
+        variable_map.put("I_0", f"{self.model_data.Options_SIGMA.simulation.conductor_current}[A]")
+        self.model.buildMPH(self.a.create_domain_array(self.g.gateway, tuple(domains)), self.input_coordinates_path,
+                            self.result_txt_path, self.generate_study, variable_map, 4, 1)
         self.model.save()
 
         with open(self.model_java_file_path) as java_file:
@@ -406,7 +413,7 @@ class BuilderSIGMA:
                                     ax.add_line(lines.Line2D([iL.x, oL.x], [iL.y, oL.y], color='red'))
 
         p.plot_multiple_areas(ax, self.coil_areas)
-        p.plot_multiple_areas(ax, self.wedge_areas, color='blue')
+        # p.plot_multiple_areas(ax, self.wedge_areas, color='blue')
         # #
         # ax.set_xlim(-self.input['air']['r'] * self.input['air']['r_ffs'],
         #             self.input['air']['r'] * self.input['air']['r_ffs'])
@@ -436,21 +443,60 @@ class BuilderSIGMA:
         script_lines += [
             f'"{self.COMSOL_compile_path}" -jdkroot "{self.java_jdk_path}" {class_paths}"{self.model_java_file_path}"',
             f'"{self.COMSOL_batch_path}" -inputfile "{self.model_class_file_path}" '
-            f'-outputfile "{os.path.join(self.output_path, self.model_data.GeneralParameters.magnet_name)}.mph"',
-            f'"{self.COMSOL_exe_path}" -open "{os.path.join(self.output_path, self.model_name)}.mph"']
+            f'-outputfile "{os.path.join(self.output_path, self.model_data.GeneralParameters.magnet_name)}.mph"']
 
         with open(self.compile_batch_file_path, "w") as outfile:
             outfile.write("\n".join(str(line) for line in script_lines))
 
 
+def create_coordinate_file(path_map2d, coordinate_file_path):
+    df = pd.read_csv(path_map2d, delim_whitespace=True)
+    df_new = pd.DataFrame()
+    df_new["X-POS/MM"] = df["X-POS/MM"].apply(lambda x: x / 1000)
+    df_new["Y-POS/MM"] = df["Y-POS/MM"].apply(lambda x: x / 1000)
+    df_new.to_csv(coordinate_file_path, header=None, index=False)
+
+
+def create_result_map2d(path_map2d, path_result, path_new_file):
+    df_reference = pd.read_csv(path_map2d, delim_whitespace=True)
+    with open(path_result) as file:  # opens a text file
+        lines = [line.strip().split() for line in file if not "%" in line]  # loops over each line
+
+    df_txt = pd.DataFrame(lines, columns=["x", "y", "Bx", "By"])
+    df_txt = df_txt.apply(pd.to_numeric)
+    with open(path_new_file, 'w') as file:
+        file.write("  BL.   COND.    NO.    X-POS/MM     Y-POS/MM    BX/T       BY/T"
+                   "      AREA/MM**2 CURRENT FILL FAC.\n\n")
+        content = []
+        for index, row in df_reference.iterrows():
+            bl, cond, no, x, y, Bx, By, area, curr, fill, fac = row
+            bl = int(bl)
+            cond = int(cond)
+            no = int(no)
+            x = f"{x:.4f}"
+            y = f"{y:.4f}"
+            Bx = df_txt["Bx"].iloc[index]
+            Bx = f"{Bx:.4f}"
+            By = df_txt["By"].iloc[index]
+            By = f"{By:.4f}"
+            area = f"{area:.4f}"
+            curr = f"{curr:.2f}"
+            fill = f"{fill:.4f}"
+            content.append(
+                "{0:>6}{1:>6}{2:>7}{3:>13}{4:>13}{5:>11}{6:>11}{7:>11}{8:>9}{9:>8}\n".format(bl, cond, no, x, y, Bx, By,
+                                                                                             area, curr, fill))
+        file.writelines(content)
+
+
 if __name__ == "__main__":
     from steam_sdk.parsers.ParserRoxie import ParserRoxie
 
-    magnet_name = 'MQXF_V2'  #'MBH_2in1'  #'MQXA'  #'MBX'  #'MQXB'   # 'MQXF_V2'  #
+    magnet_name = 'MQXA'  # 'MBH_2in1'  #'MQXA'  #'MBX'  #'MQXB'   # 'MQXF_V2'  #
     current_path = Path(__file__).parent
     path = Path.joinpath(current_path.parent.parent, 'tests', 'builders', 'model_library',
                          'magnets', magnet_name, 'input')
     file_model_data = Path.joinpath(path, 'modelData_' + magnet_name + '.yaml')
+    init_dir = os.getcwd()
     if Path.exists(file_model_data):
         with open(file_model_data, "r") as stream:
             dictionary_yaml = yaml.safe_load(stream)
@@ -465,8 +511,15 @@ if __name__ == "__main__":
         path_data = Path.joinpath(path, f"{magnet_name}.data")
     path_cadata = Path.joinpath(path.parent.parent, 'roxie.cadata')
 
+    working_dir_path = os.path.join(init_dir, magnet_name)
+    # Check if folder exists:
+    if os.path.exists(working_dir_path):
+        print("Directory already exists.")
+    else:
+        os.mkdir(working_dir_path)
     roxie_parser = ParserRoxie()
     roxie_data = roxie_parser.getData(dir_data=path_data, dir_cadata=path_cadata, dir_iron=path_iron)
+
     # with open('../tests/parsers/references/cadata_REFERENCE.yaml', 'w') as yaml_file:
     #   yaml.dump(roxie_data.conductor.dict(), yaml_file, default_flow_style=False, sort_keys=False)
     for environ_var in ['HOMEPATH', 'SWAN_HOME']:
@@ -478,4 +531,20 @@ if __name__ == "__main__":
         with open(Path.joinpath(path_settings, f"settings.{user_name}.yaml"), 'r') as stream:
             settings = yaml.safe_load(stream)
 
-    bs = BuilderSIGMA(settings_dict=settings, input_model_data=model_data, input_roxie_data=roxie_data)
+    os.chdir(working_dir_path)
+    # Create coordinate file from map2d.
+    map2d_path = os.path.join(path, f"{magnet_name}.map2d")
+    coordinate_file_path = (os.path.join(working_dir_path, f"{magnet_name}_coordinates.csv")).replace("\\", "\\\\\\\\")
+    create_coordinate_file(map2d_path, coordinate_file_path)
+    result_txt_path = (os.path.join(working_dir_path, f"{magnet_name}.txt")).replace("\\", "\\\\\\\\")
+
+    bs = BuilderSIGMA(settings_dict=settings, input_model_data=model_data, input_roxie_data=roxie_data,
+                      generate_study=model_data.Options_SIGMA.simulation.generate_study,
+                      coordinate_file_path=coordinate_file_path, result_txt_path=result_txt_path)
+    if model_data.Options_SIGMA.simulation.run_study:
+        os.system(os.path.join(working_dir_path, f"{magnet_name}_Model_Compile_and_Open.bat"))
+    create_result_map2d(map2d_path, result_txt_path, os.path.join(working_dir_path, f"{magnet_name}.map2d"))
+    if model_data.Options_SIGMA.simulation.generate_report:
+        p.plot_roxie_coil(roxie_data)
+        p.generate_report_from_map2d(os.path.join(working_dir_path, f"{magnet_name}.map2d"), f"{magnet_name}_comsol",
+                                 map2d_path, f"{magnet_name}_roxie")
