@@ -167,6 +167,31 @@ class CalculateMetrics(BaseModel):
     variables_to_analyze: List[List[str]] = [[]]
 
 
+class LoadCircuitParameters(BaseModel):
+    """
+        Level 2: Analysis step to load global circuit parameters from a .csv file
+    """
+    type: Literal['LoadCircuitParameters']
+    model_name: str = None
+    path_file_circuit_parameters: str = None
+    selected_circuit_name: str = None
+
+
+class WriteStimulusFile(BaseModel):
+    """
+        Level 2: Analysis step to write stimulus file from coil resistance csv file
+    """
+    type: Literal['WriteStimulusFile']
+    output_file: str = None
+    path_interpolation_file: Union[str, List[str]] = None
+    n_total_magnets: int = None
+    n_apertures: int = None
+    current_level: List[int] = []
+    magnets: List[int] = []
+    t_offset: List[float] = []
+    magnet_types: List[int] = []
+
+
 # class AnalysisStep(BaseModel):
 #     """
 #         Level 1: Class for information on the analysis step
@@ -178,7 +203,7 @@ class CalculateMetrics(BaseModel):
 # Highest level
 class DataAnalysis(BaseModel):
     '''
-        **Class for the circuit netlist inputs**
+        **Class for the STEAM analysis inputs**
 
         This class contains the data structure of an analysis performed with STEAM_SDK.
 
@@ -195,5 +220,5 @@ class DataAnalysis(BaseModel):
     WorkingFolders: WorkingFolders = WorkingFolders()
     AnalysisStepDefinition: Dict[str, Union[MakeModel, ModifyModel, ModifyModelMultipleVariables, SetUpFolder,
                                             AddAuxiliaryFile, CopyFile, RunSimulation, PostProcess, RunCustomPyFunction,
-                                            RunViewer, CalculateMetrics]] = {}
+                                            RunViewer, CalculateMetrics, LoadCircuitParameters, WriteStimulusFile]] = {}
     AnalysisStepSequence: List[str] = []  # Here the analysis steps are defined, in execution order. Names must be defined in AnalysisStepDefinition. Repetitions ARE allowed.
