@@ -9,6 +9,15 @@ from steam_sdk.data.DataModelMagnet import MultipoleMesh
 from steam_sdk.data.DataModelMagnet import MultipolePostProc
 
 
+class MultipoleMono(BaseModel):
+    """
+        Rutherford cable type
+    """
+    type: Literal['Mono']
+    bare_cable_width: float = None
+    bare_cable_height_mean: float = None
+
+
 class MultipoleRibbon(BaseModel):
     """
         Rutherford cable type
@@ -31,7 +40,7 @@ class MultipoleConductor(BaseModel):
     """
         Class for conductor type
     """
-    cable: Union[MultipoleRutherford, MultipoleRibbon] = {'type': 'Rutherford'}
+    cable: Union[MultipoleRutherford, MultipoleRibbon, MultipoleMono] = {'type': 'Rutherford'}
 
 
 class MultipoleGeneralSetting(BaseModel):
