@@ -59,13 +59,26 @@ class Multipole(BaseModel):
     """
     tbc: str = None
 
+class SolenoidCoil(BaseModel):
+    """
+        Level 3: Class for Solenoid windings
+    """
+    name: str = None            # -             solenoid name
+    a1: float = None            # m             smaller radial dimension of solenoid
+    a2: float = None            # m             larger radial dimension of solenoid
+    b1: float = None            # m             smaller axial dimension of solenoid
+    b2: float = None            # m             larger axial dimension of solenoid
+    conductor_name: str = None  # -             wire name - name must correspond to existing conductor name in the same yaml file
+    ntpl: int = None            # -             number of turns per layer
+    nl: int = None              # -             number of layers
+    pre_preg: float = None     # m              Pre-preg thicknes (radial) i.e. in LEDET in width direction
+    section: int = None         # Section in ledet for the block
 
 class Solenoid(BaseModel):
     """
         Level 2: Class for Solenoid windings
     """
-    tbc: str = None
-
+    coils: List[SolenoidCoil] = [SolenoidCoil()]
 
 class Pancake(BaseModel):
     """
@@ -73,7 +86,6 @@ class Pancake(BaseModel):
     """
     tbc: str = None
 
-
 class CCT_straight(BaseModel):
     """
         Level 2: Class for straight CCT windings
@@ -811,6 +823,12 @@ class SimulationSIGMA(BaseModel):
     """
         Level 2: Class for simulation parameters in SIGMA
     """
+    generate_study: bool = None
+    run_study: bool = None
+    conductor_current: float = None
+    generate_report: bool = None
+    nbr_elements_mesh_width: int = None
+    nbr_elements_mesh_height: int = None
     max_mesh_size: float = None
 
 
