@@ -140,10 +140,10 @@ def get_template_LEDET_inputs_sheet():
         ['sim3D_R_shortCircuit', None, 'Resistance of the short-circuit [Ohm] (default=1e9 Ohm)'],
         ['sim3D_shortCircuitPosition', None, 'Position of the short-circuit defined with a pair of values indicating the start and end of the conductor length that will receive the heat generated in the short-circuit [m]. For a simulation without short-circuit, set to the scalar 0 (default=0)'],
         ['sim3D_durationGIF', None, 'Duration of the animated GIF [s]'],
-        ['sim3D_flag_saveFigures', None, 'Save figures showing the main results'],
-        ['sim3D_flag_saveGIF', None, 'Save GIF showing the main results'],
+        ['sim3D_flag_saveFigures', None, 'If set to 1, save figures showing the main results'],
+        ['sim3D_flag_saveGIF', None, 'If set to 1, save GIF showing the main results'],
         ['sim3D_flag_VisualizeGeometry3D', None, 'Visualize the 3D geometry'],
-        ['sim3D_flag_SaveGeometry3D', None, 'Save figure showing the 3D Geometry'],
+        ['sim3D_flag_SaveGeometry3D', None, 'If set to 1, save figures of the 3D geometry (figures identified by nameMagnet). If set to 2, save figures of the 3D geometry (figures identified by nameMagnet and simToRun).'],
         [None, None, None],
         [None, None, 'Magnet inductance'],
         ['M_m', None, 'Self mutual inductance matrix [H/m]'],
@@ -172,14 +172,15 @@ def get_template_LEDET_options_sheet():
         ['columnsXY', None, 'Columns containing the X and Y positions of each strand in the map files (positions given in mm)'],
         ['columnsBxBy', None, 'Columns containing the magnetic field in the X and Y direction at the reference current for each strand in the map files (magnetic field given in T)'],
         ['flagPlotMTF', None, 'Plot magnetic field distributions for each coil section (=1), or not (=0)'],
-        ['fieldMapNumber', None, 'Number defining a set of magnetic-field maps, whose names should end with a suffix "_N", where N is  the set number'],
-        ['flag_calculateMagneticField', None, 'If set to 1, the 2D magnetic-field maps will be calculated (not imported). The calculation will rely on the Biot-Savart law and use the strand positions imported from the map2d file.'],
+        ['fieldMapNumber', None, '[Optional] Number defining a set of magnetic-field maps, whose names should end with a suffix "_N", where N is  the set number'],
+        ['flag_calculateMagneticField', None, 'If set to 1, the 2D magnetic-field maps will be calculated (not imported). The calculation will rely on the Biot-Savart law and use the strand positions imported from the map2d file. (default=0)'],
         [None, None, None],
         [None, None, 'Input Generation Options'],
         ['flag_typeWindings', None, 'Coil windings type. 0=Multipole coil. 1=Solenoid. 2=CCT coil. 101=Busbar. Default: 0'],
         ['flag_calculateInductanceMatrix', None, 'If set to >0, enables the automatic calculation of the self- and mutual inductances between each turn, and between each coil section. If set to 2, it also write an output txt file with the calculated values; this file is saved in the subfolder "Output\Txt Files".'],
         ['flag_useExternalInitialization', None, 'If set to 1, LEDET will look for an initialization file in a specified folder to load all input properties and magnetic field parameters; if the file is not there, it will generate it (default=0)'],
         ['flag_initializeVar', None, 'Initialize selected variables from a txt files. The variables are defined in the "Variables" sheet'],
+        ['selfMutualInductanceFileNumber', None, '[Optional] Number defining a version of the self-mutual inductance .csv file, whose name should be nameMagnet+"_selfMutualInductanceMatrix_"+N+".csv" where N is  the set number. Setting this value to 0 will disable this option (not reading a file with N=0)'],
         [None, None, None],
         [None, None, 'Input Simulation Options'],
         ['flag_fastMode', None, 'If set to 1, run the simulation in fast mode: faster and less heavy (about 75% less RAM needed), but not all variables are saved (RECOMMENDED)'],
@@ -268,6 +269,6 @@ def get_template_LEDET_variables_sheet():
         ['writeToMesh_suffixFileNameOutput', None, 'Name of the suffix of the output files. Full names will be magnet_name + suffix + simulation_number + time_step + ".RES"'],
         ['writeToMesh_selectedVariables', None, 'Selected variables that will be written to the mesh elements. More than one can be defined in each column (comma separated).'],
         ['writeToMesh_selectedTimeSteps', None, 'Selected time steps at which the results will be written to the mesh elements'],
-        ['writeToMesh_selectedMethod', None, 'Method for assigning variables to nodes. 1=Assign to each mesh node the value of the closest strand (good for distributed properties like temperature).2=Assign the value of each strand to the closest node (good for electromagnetic forces)'],
+        ['writeToMesh_selectedMethod', None, 'Method for assigning variables to nodes. 1=Assign to each mesh node the value of the closest strand (good for distributed properties like temperature). 2=Assign the value of each strand to the closest node (good for electromagnetic forces)'],
     ]
     return template_LEDET_variables_sheet
\ No newline at end of file
