@@ -48,4 +48,5 @@ class DriverPSPICE:
         else:
             self.output = subprocess.call([path_exe, full_name_file], stdout=subprocess.DEVNULL, stderr=subprocess.DEVNULL)
 
-
+        if self.output:
+            raise Exception('PSPICE failed to run successfully.')
