@@ -4,6 +4,8 @@ import ntpath
 import shutil
 import textwrap
 from pathlib import Path
+from typing import Union
+
 import matplotlib.pyplot as plt
 from scipy.interpolate import UnivariateSpline
 from scipy.interpolate import interp1d
@@ -1076,3 +1078,130 @@ def writeStimuliFromInterpolation(current_level: list, n_total_magnets: int, n_a
     # Write the stimuli as a txt(stl) file
     with open(Outputfile, type_stl) as ofile:
         ofile.write(stlString)
+
+
+
+def writeStimuliFromInterpolation_general(current_level: list, n_total_magnets: int, n_apertures: int, magnets: list, tShift: list, Outputfile: str, path_resources: Union[str,list],
+                                  InterpolationType: str = 'Linear', type_stl: str = 'a', sparseTimeStepping: int = 100, magnet_type: list = None):
+    '''
+    Function to write a resistance stimuli for n apertures of a magnet for any current level. Resistance will be interpolated
+    from pre-calculated values (see InterpolateResistance for closer explanation). Stimuli is then written in a .stl file for PSPICE
+
+    :param current_level: list, all current level that shall be used for interpolation (each magnet has 1 current level)
+    :param n_total_magnets: int, Number of total magnets in the circuit (A stimuli will be written for each, non-quenching = 0)
+    :param n_apertures: int, Number of apertures per magnet. A stimuli will be written for each aperture for each magnet
+    :param magnets: list, magnet numbers for which the stimuli shall be written
+    :param tShift: list, time shift that needs to be applied to each stimuli
+    (e.g. if magnet 1 quenches at 0.05s, magnet 2 at 1s etc.), so that the stimuli are applied at the correct time in the simulation
+    :param Outputfile: str, name of the stimuli-file
+    :param path_resources: list, list of paths to the files with pre-calculated values
+    :param magnet_type: list, contains numbers from 1 to len(path_resources) for each magnet
+    (e.g. if there are three items in the path_resources list and four magnets, then magnet_type can be [2,2,1,1] or [1,2,3,3] telling us which path_resource file to use for each magnet
+    :param Type: str, either Linear or Spline, type of interpolation
+    :param type_stl: str, how to write the stimuli file (either 'a' (append) or 'w' (write))
+    :param sparseTimeStepping: int, every x-th time value only a stimuli point is written (to reduce size of stimuli)
+    :return:
+    '''
+    # Check inputs and set default vaues
+    if type(path_resources) == str:
+        path_resources = [path_resources] * len(magnets)  # make a list out of the string
+
+    if not magnet_type:
+        magnet_type = [1] * len(magnets)  # make a list of 1's (to maintain the same behavior as the older version fo the function)
+
+    if n_apertures>2: raise Exception('Maximum of 2 apertures supported yet.')
+    # Ensure consistency of inputs
+    if len(magnets) != len(tShift): raise Exception(f'Please provide a time shift for each magnet. '
+                                                    f'Size of magnet list is {len(magnets)}, but size of tShift is {len(tShift)}')
+    if len(magnet_type) != len(magnets): raise Exception(f'Please provide a magnet type for interpolation of each magnet.'
+                                                         f'Size of magnet_type is {len(magnet_type)}, but size of magnet list is {len(magnets)}')
+    R1 = np.array([])
+    R2 = np.array([])
+    print("Interpolating Coil-Resistances")
+    stlString = ''
+    # Interpolate data for each current level and ensure correct data format
+    for k in range(1, n_total_magnets + 1):
+        for l in current_level:
+            if n_apertures == 2:
+                [time, data_R1, data_R2] = InterpolateResistance(l, path_resources[k-1], n_apertures=n_apertures, Type = InterpolationType)
+            else:
+                [time, data_R1] = InterpolateResistance(l, path_resources[k-1], n_apertures=n_apertures, Type=InterpolationType)
+            if not R1.size > 0:
+                R1 = data_R1[np.newaxis, ...]
+            else:
+                R1 = np.vstack((R1, data_R1))
+            if n_apertures == 2:
+                if not R2.size > 0:
+                    R2 = data_R2[np.newaxis, ...]
+                else:
+                    R2 = np.vstack((R2, data_R2))
+        #stlString = ''
+
+    # Loop through all current level
+    #for k in range(1, n_total_magnets+1):
+        if k in magnets:
+            index = magnets.index(k)
+            timeShift = tShift[index]
+            if timeShift < 0: timeShift = 0
+
+            # Start with generating stimulus for first aperture
+            stlString = stlString + f'\n.STIMULUS R_coil_{str(1)}_M{str(magnets[index])} ' \
+                                    f'PWL \n+ TIME_SCALE_FACTOR = 1 \n+ VALUE_SCALE_FACTOR = 1 \n'
+            stlString = stlString + "+ ( 0s, 0.0 )\n"
+            count = 0
+            # Set a resistance value in the stimuli for each sparseTimeStepping * TimeValue
+            for l in range(1, R1.shape[1] - 1):
+                if np.isnan(R1[index, l]): continue
+                # Ensure starting at 0 (No negative time in PSPICE allowed
+                if float(time[l]) < 0:  timeShift = timeShift + abs(float(time[l])) - 0.03
+                if float(time[l]) + timeShift < 0:
+                    tt = 0
+                else:
+                    tt = float(time[l]) + timeShift
+                # If every sparseTimeStepping* time value reached, write an entry
+                if count >= sparseTimeStepping:
+                    stlString = stlString + "+ ( " + str(tt) + "s, " + str(R1[index, l]) + " )\n"
+                    count = 0
+                count = count + 1
+            # Write last time value to be 10000 s, with last resistance (resistance is assumed to stay constant)
+            R1_last = R1[index]
+            R1_last = R1_last[~np.isnan(R1_last)]
+            stlString = stlString + "+ ( " + str(10000) + "s," + str(R1_last[-1]) + " ) \n"
+            stlString = stlString + " \n"
+
+            if n_apertures == 2:
+                # Repeat procedure for second aperture
+                stlString = stlString + f'\n.STIMULUS R_coil_{str(2)}_M{str(magnets[index])} ' \
+                                        f'PWL \n+ TIME_SCALE_FACTOR = 1 \n+ VALUE_SCALE_FACTOR = 1 \n'
+                stlString = stlString + "+ ( 0s, 0.0 )\n"
+                count = 0
+                for l in range(1, R2.shape[1] - 1):
+                    if np.isnan(R2[index, l]): continue
+                    if float(time[l]) < 0:  timeShift = timeShift + abs(float(time[l])) - 0.03
+                    if float(time[l]) + timeShift < 0:
+                        tt = 0
+                    else:
+                        tt = float(time[l]) + timeShift
+                    if count >= sparseTimeStepping:
+                        stlString = stlString + "+ ( " + str(tt) + "s, " + str(R2[index, l]) + " )\n"
+                        count = 0
+                    count = count + 1
+                R2_last = R2[index]
+                R2_last = R2_last[~np.isnan(R2_last)]
+                stlString = stlString + "+ ( " + str(10000) + "s," + str(R2_last[-1]) + " ) \n"
+                stlString = stlString + " \n"
+        else:
+            stlString = stlString + f'\n.STIMULUS R_coil_{str(1)}_M{str(k)} ' \
+                                    f'PWL \n+ TIME_SCALE_FACTOR = 1 \n+ VALUE_SCALE_FACTOR = 1 \n'
+            stlString = stlString + "+ ( 0s, 0.0 )\n"
+            stlString = stlString + "+ ( 10000s, 0.0 )\n"
+
+            if n_apertures == 2:
+                stlString = stlString + f'\n.STIMULUS R_coil_{str(2)}_M{str(k)} ' \
+                                        f'PWL \n+ TIME_SCALE_FACTOR = 1 \n+ VALUE_SCALE_FACTOR = 1 \n'
+                stlString = stlString + "+ ( 0s, 0.0 )\n"
+                stlString = stlString + "+ ( 10000s, 0.0 )\n"
+
+    # Write the stimuli as a txt(stl) file
+    with open(Outputfile, type_stl) as ofile:
+        ofile.write(stlString)
