@@ -156,7 +156,7 @@ class ParserXYCE:
 
                 # Read print
                 if '.PRINT' in line:
-                    format = line.split('=')[-1]
+                    format = line.split('=')[1].split(' ')[0]
                     if format=='PROBE':
                         self.circuit_data.PostProcess.probe.probe_type = 'standard'
                     elif format=='CSV':
@@ -389,9 +389,9 @@ class ParserXYCE:
             rows_global_parameters.append('.PARAM')  # Add header of this section
             # Add global parameter entries
             for name, value in self.circuit_data.GlobalParameters.global_parameters.items():
-                if '#' in value:
+                if '#' in str(value):
                     value = value.replace('#','')
-                if '#' in name:
+                if '#' in str(name):
                     name = name.replace('#','')
                 rows_global_parameters.append(add_global_parameter(name, value))
 
@@ -525,7 +525,8 @@ class ParserXYCE:
         probe_type = self.circuit_data.PostProcess.probe.probe_type
         probe_variables = self.circuit_data.PostProcess.probe.variables
         if probe_type:
-            rows_post_processing.append(add_probe(probe_type, probe_variables, self.circuit_data.Analysis.analysis_type))
+            rows_post_processing.append(add_probe(probe_type, probe_variables, self.circuit_data.Analysis.analysis_type,
+                                                  os.path.join(current_path, self.circuit_data.GeneralParameters.circuit_name)))
 
         # Prepare file end
         rows_file_end = [add_end_file()]
@@ -783,14 +784,14 @@ def add_transient_time_schedule(time_schedule):
     return formatted_text
 
 
-def add_probe(probe_type: str, probe_variables: list, analysis_type: str):
+def add_probe(probe_type: str, probe_variables: list, analysis_type: str, csd_path: str):
     ''' Format probe row '''
     if analysis_type=='frequency':
         type = 'AC'
     elif analysis_type=='transient':
         type = 'TRAN'
     if probe_type == 'standard' or probe_type == 'CSDF':
-        formatted_text = f'.PRINT {type} FORMAT=PROBE'
+        formatted_text = f'.PRINT {type} FORMAT=PROBE' + f' FILE={csd_path}.csd'
     else:
         raise Exception('Probe entry has an unknown or not implemented type: {}.'.format(probe_type))
 
