@@ -22,16 +22,21 @@ def yaml_to_data(full_file_path, data_class=dict, with_comments=False):
 
 def dict_to_yaml(data_dict, name_output_file: str, list_exceptions: list = [], with_comments=False):
     """
-        Write a dictionary to yaml with pre-set format used across STEAM yaml files.
-        In particular:
-        - keys order is preserved
-        - lists are written in a single row
-        :param data_dict: Dictionary to write
-        :param name_output_file: Name of the yaml file to write
-        :param list_exceptions: List of strings defining keys that will not be written in a single row
-        :param with_comments: If true, preserves comments in the yaml file on save. Needs to be used in combination with yaml_to_data_class flag with_comments=True
+    Write a dictionary to yaml with pre-set format used across STEAM yaml files.
+    In particular:
+    - keys order is preserved
+    - lists are written in a single row
+    :param data_dict: python dictionary to write
+    :type data_dict: dict
+    :param name_output_file: Full path to yaml file to write
+    :type name_output_file: str
+    :param list_exceptions: List of strings defining keys that will not be written in a single row
+    :type list_exceptions: list
+    :param with_comments: If true, preserves comments in the yaml file on save. Needs to be used in combination with yaml_to_data_class flag with_comments=True
+    :type with_comments: bool
+    :return: Nothing, writes yaml file to disc
+    :rtype: None
     """
-
     #################################################################################################
     # Helper functions
     def my_represent_none(obj, *args):
