@@ -5,9 +5,11 @@ import numpy as np
 import pandas as pd
 import yaml
 
-import matplotlib.pyplot as plt
 from steam_sdk.data.DataSignal import DataSignal
 
+import matplotlib
+import matplotlib.pyplot as plt
+
 from steam_sdk.parsers.ParserMat import get_signals_from_mat
 from steam_sdk.parsers.ParserCsv import get_signals_from_csv
 from steam_sdk.parsers.ParserCsd import get_signals_from_csd
@@ -379,6 +381,8 @@ class Viewer:
         default_figsize = (12, 10)
 
         # Plot - Circuit current
+        if flag_display:
+            matplotlib.use('Agg')  # using Agg backend so that figures will not be displayed when not wanted
         fig, ax = plt.subplots(1, 1, figsize=default_figsize)
         if meas_label:
             plt.plot(x_meas, y_meas, '.-', linewidth=1.0, color='r', label=meas_label)
@@ -393,11 +397,16 @@ class Viewer:
         plt.grid(True)
         plt.legend(loc='best')
         # plt.tight_layout()
-        if flag_display:
-            plt.show(block=True)  # show this figure
         if full_path_figure_to_save:
             plt.savefig(full_path_figure_to_save, format='png')  # Save figure
-        return fig, ax
+        if flag_display:
+            plt.show(block=True)  # show this figure
+        else:
+            plt.plot(block=False)
+            fig.clf()
+            plt.clf()  # clears the figure with all the axis in it (leaves window still open)
+            plt.close()  # closes the window
+        # return fig, ax   # NOTE: return value not needed... if needed figure has to be cleared at one time
 
 
     # def plot_ViewerSim(self, data, titles, labels, types, texts, size, legends, style, window, scale, order):
